<?php
/**
 * @author Tomáš Blatný
 */

namespace Axima\Sms\Cron;


use Axima\Sms\Utils\Database;
use Axima\SmsGate\Client as SmsClient;
use DateTime;
use GuzzleHttp\Client;
use Magento\Framework\App\ObjectManager;


class CheckJob
{

	const CONFIG_PATH = __DIR__ . '/../../../../../var/config_axima_sms.json';


	public function execute()
	{
		$objectManager = ObjectManager::getInstance();
		$database = new Database($objectManager->get('\Magento\Framework\App\ResourceConnection'));
		if ($this->isClientSet()) {
			$client = $this->getSmsClient();
			$reports = $client->getDeliveryReports();
			foreach ($reports as $id => $timestamp) {
				$date = date_create_from_format('YmdGis', $timestamp);
				$database->update('axima-sms-messages', array('status' => 1, 'date_received' => $date->format('Y-m-d G:i:s')), '`sms_id` = %s', array($id));
				$client->confirmDeliveryReport($id);
			}
			$date = new DateTime;
			$this->updateOptions(array('lastCheck' => $date->format('Y-m-d G:i:s')));
		}
	}

	/**
	 * @return array
	 */
	private function getOptions()
	{
		$config = @file_get_contents(self::CONFIG_PATH);
		if (!$config) {
			$config = array();
		} else {
			$config = json_decode($config, JSON_OBJECT_AS_ARRAY);
		}
		return $config;
	}


	/**
	 * @param string $name
	 * @param mixed $default
	 * @return mixed
	 */
	private function getOption($name, $default = NULL)
	{
		$options = $this->getOptions();
		return isset($options[$name]) ? $options[$name] : $default;
	}


	private function updateOptions(array $options)
	{
		$current = $this->getOptions();
		foreach ($options as $key => $option) {
			$current[$key] = $option;
		}
		file_put_contents(self::CONFIG_PATH, json_encode($current));
	}


	private function isClientSet()
	{
		return $this->getOption('name') !== NULL && $this->getOption('password') !== NULL;
	}


	private function getSmsClient()
	{
		return new SmsClient(new Client, $this->getOption('name'), $this->getOption('password'));
	}

}
