<?php
/**
 * @author Tomáš Blatný
 */

namespace Axima\Sms\Model\Plugin;


use Axima\Sms\Utils\Database;
use Axima\SmsGate\Client as SmsClient;
use Axima\SmsGate\Exception;
use Axima\SmsGate\SmsGateException;
use DateTime;
use GuzzleHttp\Client;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Event\Observer as MagentoObserver;
use Magento\Framework\Event\ObserverInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;

class Observer implements ObserverInterface
{

	const TABLE_MESSAGES = 'axima_sms_messages';
	const CONFIG_PATH = __DIR__ . '/../../../../../../var/config_axima_sms.json';

	const STATUS_SENT = 0;
	const STATUS_DELIVERED = 1;
	const STATUS_ERROR = 2;

	/** @var ObjectManager */
	private $objectManager;

	/** @var Database */
	private $database;

	public function execute(MagentoObserver $observer)
	{
		/** @var OrderInterface $result */
		$result = $observer->getData('order');
		//file_put_contents(__DIR__ . '/../../log.txt', date('G:i:s j.n.Y') . ' ' . $result->getEntityId() . ' ' . $result->getStatus() . "\n", FILE_APPEND);

		$this->objectManager = ObjectManager::getInstance();
		$this->database = new Database($this->objectManager->get('\Magento\Framework\App\ResourceConnection'));

		$newStatus = $result->getStatus();

		$codes = $this->getShortCodes($result);

		$texts = $this->getOption('texts', array());
		$enabled = $this->getOption('enabled', array());
		if (isset($enabled[$newStatus], $texts[$newStatus]) && $enabled[$newStatus] && trim($texts[$newStatus])) {
			$phone = str_replace(' ', '', $result->getBillingAddress()->getTelephone());
			/*$log = */$text = trim($texts[$newStatus]);

			$text = str_replace(array_map(function ($item) {
				return '[' . $item . ']';
			}, array_keys($codes)), array_values($codes), $text);

			//$log .= "\n" . $text;
			//$log .= "\n" . print_r($codes, TRUE) . "\n";
			//file_put_contents(__DIR__ . '/../../logText.txt', $log, FILE_APPEND);

			$this->sendSms($phone, $text);
		}

		return $result;
	}


	private function getShortCodes(OrderInterface $order = NULL)
	{
		if (!$order) {
			return array( // dummy data
				'name' => 'John Doe',
				'billingAddress' => 'Narrow Street 145, London, B45 0HY',
				'price' => '1785 EUR',
				'paymentMethod' => 'Bank transfer',
			);
		}
		return array(
			'name' => $order->getBillingAddress()->getFirstname() . ' ' . $order->getBillingAddress()->getLastname(),
			'billingAddress' => implode(' ', $order->getBillingAddress()->getStreet()) . ' ' . $order->getBillingAddress()->getCity() . ', ' . $order->getBillingAddress()->getPostcode(),
			'price' => round($order->getGrandTotal()) . ' ' . $order->getOrderCurrencyCode(),
			'paymentMethod' => $order->getPayment()->getMethod(),
		);
	}


	private function sendSms($number, $text)
	{
		$client = $this->getSmsClient();
		$date = new DateTime;
		$date = $date->format('Y-m-d G:i:s');
		try {
			$result = $client->sendSms($text, $number, TRUE);
			$this->database->insert(self::TABLE_MESSAGES, array(
				'sms_id' => $result['id'],
				'text' => $text,
				'number' => $number,
				'status' => self::STATUS_SENT,
				'date_sent' => $date,
			));
			$this->updateOptions(array(
				'credit' => $result['credit'],
				'credit_last_update' => $date,
			));
		} catch (Exception $exception) {
			$note = '';
			if ($exception instanceof SmsGateException) {
				$note = $exception->getMessage();
			}
			$this->database->insert(self::TABLE_MESSAGES, array(
				'sms_id' => NULL,
				'text' => $text,
				'number' => $number,
				'status' => self::STATUS_ERROR,
				'note' => $note,
				'date_sent' => $date,
			));
		}

	}


	private function getSmsClient()
	{
		return new SmsClient(new Client, $this->getOption('name'), $this->getOption('password'));
	}


	/**
	 * @return array
	 */
	private function getOptions()
	{
		$config = @file_get_contents(self::CONFIG_PATH);
		if (!$config) {
			$config = array();
		} else {
			$config = json_decode($config, JSON_OBJECT_AS_ARRAY);
		}
		return $config;
	}


	/**
	 * @param string $name
	 * @param mixed $default
	 * @return mixed
	 */
	private function getOption($name, $default = NULL)
	{
		$options = $this->getOptions();
		return isset($options[$name]) ? $options[$name] : $default;
	}

	private function updateOptions(array $options)
	{
		$current = $this->getOptions();
		foreach ($options as $key => $option) {
			$current[$key] = $option;
		}
		file_put_contents(self::CONFIG_PATH, json_encode($current));
	}
}
