<?php
/**
 * @author Tomáš Blatný
 */

namespace Axima\Sms\Setup;


use Magento\Framework\DB\Ddl\Table;
use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

class InstallSchema implements InstallSchemaInterface
{

	public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
	{
		$installer = $setup;
		$data = array(
			'enabled' => array(
				'processing' => TRUE,
				'complete' => TRUE,
			),
			'texts' => array(
				'processing' => __('Thank you, your order is being processed.'),
				'complete' => __('Thank you, your order has been completed.'),
			),
		);
		@file_put_contents(__DIR__ . '/../../../../../var/config_axima_sms.json', json_encode($data));

		$installer->startSetup();

		$tableName = $installer->getTable('axima_sms_messages');
		if (!$installer->getConnection()->isTableExists($tableName)) {
			$table = $installer->getConnection()
				->newTable($tableName)
				->addColumn('id', Table::TYPE_INTEGER, NULL, [
					'identity' => TRUE,
					'unsigned' => TRUE,
					'nullable' => FALSE,
					'primary' => TRUE,
				])
				->addColumn('sms_id', Table::TYPE_TEXT, NULL, ['nullable' => TRUE, 'default' => NULL])
				->addColumn('text', Table::TYPE_TEXT, NULL, ['nullable' => FALSE])
				->addColumn('number', Table::TYPE_TEXT, NULL, ['nullable' => FALSE])
				->addColumn('status', Table::TYPE_SMALLINT, NULL, ['nullable' => FALSE, 'unsigned' => TRUE])
				->addColumn('note', Table::TYPE_TEXT, NULL, ['nullable' => TRUE, 'default' => NULL])
				->addColumn('date_sent', Table::TYPE_DATETIME, NULL, ['nullable' => FALSE])
				->addColumn('date_received', Table::TYPE_DATETIME, NULL, ['nullable' => TRUE])
				->setOption('type', 'InnoDB')
				->setOption('charset', 'utf8');
			$installer->getConnection()->createTable($table);
		}

		$installer->endSetup();
	}

}
